%--------------------------------------------------------%
%  Assemblage des matrices de masse et de rigidite       %
%  et du second membre pour methode d'elements finis P1  %
%  suivant le domaine physique considere                 %
%                                                        %
%  C. Besse, IMT Toulouse, 2018                          %
%--------------------------------------------------------%
function [M,R,rhs]=assemblage_dom_phys(m,dom_phys)
  ind=find(m.TRIANGLES(:,4)==dom_phys);

  M=sparse(m.nbNod,m.nbNod);
  R=sparse(m.nbNod,m.nbNod);
% $$$   M=spdiags([ones(m.nbNod,1)],0,m.nbNod,m.nbNod);
% $$$   R=spdiags([ones(m.nbNod,1)],0,m.nbNod,m.nbNod);
  rhs=zeros(m.nbNod,1);
  
  K=zeros(m.nbNod,1);
  
  for ll=1:length(ind)
    l=ind(ll);
    %reperer les points du triangle
    k1=m.TRIANGLES(l,1); %correspond au point A1
    k2=m.TRIANGLES(l,2); %correspond au point A2
    k3=m.TRIANGLES(l,3); %correspond au point A3

    K(k1)=1; K(k2)=1; K(k3)=1;
    
    %relever les coordonnees des points
    x1=m.POS(k1,1); y1=m.POS(k1,2); %coordonnees du point A1
    x2=m.POS(k2,1); y2=m.POS(k2,2); %coordonnees du point A2
    x3=m.POS(k3,1); y3=m.POS(k3,2); %coordonnees du point A3

    %calcul de l'aire du triangle
    aire=det([x2-x1,x3-x1;y2-y1,y3-y1])/2;
    
    %creation de la matrice de rigidite locale
    Kloc=zeros(3,3);
    J=[y3-y1,y1-y2;x1-x3,x2-x1];
    lambda=[-1,1,0;-1,0,1];
    for j=1:3
        Kloc(1,j)=(J*lambda(1:2,j))'*(J*lambda(1:2,1));
        Kloc(2,j)=(J*lambda(1:2,j))'*(J*lambda(1:2,2));
        Kloc(3,j)=(J*lambda(1:2,j))'*(J*lambda(1:2,3));
    end
    Kloc=1/(4*aire)*Kloc;

    %remplissage de la matrice de rigidite
    %R(k1,k1)=0; R(k2,k2)=0; R(k3,k3)=0;
    R(k1,k1)=R(k1,k1)+Kloc(1,1);    R(k1,k2)=R(k1,k2)+Kloc(1,2);    R(k1,k3)=R(k1,k3)+Kloc(1,3);
    R(k2,k1)=R(k2,k1)+Kloc(2,1);    R(k2,k2)=R(k2,k2)+Kloc(2,2);    R(k2,k3)=R(k2,k3)+Kloc(2,3);
    R(k3,k1)=R(k3,k1)+Kloc(3,1);    R(k3,k2)=R(k3,k2)+Kloc(3,2);    R(k3,k3)=R(k3,k3)+Kloc(3,3);

    %creation de la matrice de masse locale
    Mloc=(aire/12)*[2,1,1;1,2,1;1,1,2];
    
    %remplissage de la matrice de masse
    %M(k1,k1)=0; M(k2,k2)=0; M(k3,k3)=0;
    M(k1,k1)=M(k1,k1)+Mloc(1,1);    M(k1,k2)=M(k1,k2)+Mloc(1,2);    M(k1,k3)=M(k1,k3)+Mloc(1,3);
    M(k2,k1)=M(k2,k1)+Mloc(2,1);    M(k2,k2)=M(k2,k2)+Mloc(2,2);    M(k2,k3)=M(k2,k3)+Mloc(2,3);
    M(k3,k1)=M(k3,k1)+Mloc(3,1);    M(k3,k2)=M(k3,k2)+Mloc(3,2);    M(k3,k3)=M(k3,k3)+Mloc(3,3);

    %remplissage du second membre
    rhs([k1;k2;k3])=rhs([k1;k2;k3])+Mloc*[fun_s(x1,y1);fun_s(x2,y2);fun_s(x3,y3)];
  end

  for kk=1:m.nbNod
    if not(K(kk))
      R(kk,kk)=1;
      M(kk,kk)=1;
    end
  end