%--------------------------------------------------------%
%  Resolution de l'equation de Poisson par methode       %
%                d'elements finis                        %
%                                                        %
%  Conditions aux limites de Dirichlet homogenes         %
%                                                        %
%  Resolution par decomposition de domaines              %
%  Methode de Schwarz                                    %
%                                                        %
%  C. Besse, IMT Toulouse, 2018                          %
%--------------------------------------------------------%

clear all;
close all;

%m=load_gmsh('Maillages/ef2.msh');
%m=load_gmsh('Maillages/ef.msh');
m=load_gmsh('Maillages/cercle.msh');

param=10;

%% Representation des maillages pour les deux domaines
figure(1);
p1=trace_maillage_phys(m,1000);
p2=trace_maillage_phys(m,2000);
set(p1,'EdgeColor',[0 0 1]); %p1.EdgeColor=[0 0 1];
set(p2,'EdgeColor',[1 0 0]); %p2.EdgeColor=[1 0 0];
axis equal;

u1n=zeros(m.nbNod,1);
u2n=u1n;

% Zone d'echange frontiere :
% reperes 1002 (dom 1) et 2004 (dom 2)
ind_rep_bc_dom1=find(m.LINES(:,3)==1002);
ind_rep_bc_dom2=find(m.LINES(:,3)==2004);

% Preparation des premieres valeurs a la frontiere de transmission
% 0 ou aleatoire
for jj=1:length(ind_rep_bc_dom1)
  P1=m.LINES(ind_rep_bc_dom1(jj),1);
  P2=m.LINES(ind_rep_bc_dom1(jj),2);
  u1n([P1;P2])=0*rand(2,1);
end
for jj=1:length(ind_rep_bc_dom2)
  P1=m.LINES(ind_rep_bc_dom2(jj),1);
  P2=m.LINES(ind_rep_bc_dom2(jj),2);
  u2n([P1;P2])=0*rand(2,1);
end

%% Trace des solutions sur les deux domaines
%% En bleu, domaine 1, en rouge, domaine 2
figure(2);
p1=trace_sol_phys(m,u1n,1000);
p2=trace_sol_phys(m,u2n,2000);
colormap('gray')
set(p1,'EdgeColor',[0 0 1]); %p1.EdgeColor=[0 0 1]; 
set(p2,'EdgeColor',[1 0 0]); %p2.EdgeColor=[1 0 0]; 
set(p1,'EdgeAlpha',0.2);     %p1.EdgeAlpha=0.2;     
set(p2,'EdgeAlpha',0.2);     %p2.EdgeAlpha=0.2;     

%% Preparation des tableaux pour le stockage des 
%% derivees normales
dnu1n=zeros(m.nbNod,1);
dnu2n=zeros(m.nbNod,1);


% Assemblage des matrices de masse, de rigite
% et du second membre (depend de la fonction fun_s.m)
[M1,R1,rhs1]=assemblage_dom_phys(m,1000);
%A1=M1+R1;
A1=R1;
    
% Assemblage des matrices de masse, de rigite
% et du second membre (depend de la fonction fun_s.m)
[M2,R2,rhs2]=assemblage_dom_phys(m,2000);
%A2=M2+R2;
A2=R2;

% Prise en compte des conditions aux limites de Dirichlet
[A1,rhs1]=cond_lim_dirichlet(m,A1,rhs1,0,[1001;1003;1004]);
[A2,rhs2]=cond_lim_dirichlet(m,A2,rhs2,0,[2001;2002;2003]);

% Prise en compte des conditions de transmission aux interfaces
[A1]=cond_lim_dom_phys2(m,1002,A1,param);
[A2]=cond_lim_dom_phys2(m,2004,A2,param);

for jjj=1:20
  dnu1=dnu1n;
  dnu2=dnu2n;
  u1=u1n;
  u2=u2n;
  [rhs1b]=cond_lim_rhs(m,1002,rhs1,param,u2,dnu2);
  [rhs2b]=cond_lim_rhs(m,2004,rhs2,param,u1,-dnu1);

  u1n=A1\rhs1b;
  u2n=A2\rhs2b;

  [dnu1n]=update_dnu2(m,1002,param,u1n,u2,dnu2);
  [dnu2n]=update_dnu2(m,2004,-param,u2n,u1,dnu1);
  
  figure(2);
  clf;
  p1=trace_sol_phys(m,u1n,1000);
  p2=trace_sol_phys(m,u2n,2000);
  title(strcat('Etape ',num2str(jjj)));
  colormap('gray')
  set(p1,'EdgeColor',[0 0 1]);   %p1.EdgeColor=[0 0 1];
  set(p2,'EdgeColor',[1 0 0]);   %p2.EdgeColor=[1 0 0];
  set(p1,'EdgeAlpha',0.5);       %p1.EdgeAlpha=0.2;    
  set(p2,'EdgeAlpha',0.5);       %p2.EdgeAlpha=0.2;    

  drawnow;
  pause;
end
