%--------------------------------------------------------%
%  Resolution de l'equation d'advection diffusion        %
%                                                        %
%  \partial_t u + a\partial_x u -\nu \partial_x^2 u =f   %
%  Conditions aux limites de Dirichlet a gauche,         %
%  Neumann a droite                                      %
%                                                        %
%  C. Besse, IMT Toulouse, 2018                          %
%--------------------------------------------------------%


clear all; close all;

%% Donnees du probleme
a0=-5; b0=5;
T=1;
L=b0-a0;
N=1000;
dx=L/N;
x=(a0:dx:b0)';
Niter=100;
dt=T/Niter;

a=1;
nu=1;

%% Assemblage de la matrice
e=ones(N+1,1);
Mt=(2/dt)*spdiags([e],0,N+1,N+1);
Ma=(0.5/dx)*spdiags([-e,e],[-1,1],N+1,N+1);
Mlap=(1/dx^2)*spdiags([e,-2*e,e],[-1:1],N+1,N+1);

M=Mt+a*Ma-nu*Mlap;

%% Donnee initiale
phi=exp(-x.^2);

%% Terme source
f=0.5*(x>-1).*(x<1);

% Conditions aux limites
cl=zeros(N+1,1);

% Dirichlet a gauche
gl=0;
M(1,1)=1;
M(1,2)=0;
cl(1)=gl;

% Neumann a droite
gr=0;
M(N+1,N+1)=2/dt+2*nu/dx^2;
M(N+1,N)=-2*nu/dx^2;
cl(N+1)=(2*nu/dx-a)*gr;

[LL,UU]=lu(M);

% Resolution du probleme temporel
for j=1:Niter-1
  rhs=f+cl+(2/dt)*phi; rhs(1)=cl(1);
  y=LL\rhs;
  v=UU\y;
  phi=2*v-phi; phi(1)=gl;
  
  plot(x,phi);axis([a0,b0,0,1.2]);
  drawnow;
  %pause;
end
