%--------------------------------------------------------%
%  Resolution de l'equation d'advection diffusion        %
%                                                        %
%  \partial_t u + a\partial_x u -\nu \partial_x^2 u =f   %
%  Conditions aux limites de Dirichlet a gauche,         %
%  Neumann a droite                                      %
%                                                        %
%  Resolution par decomposition de domaines              %
%  Methode de Schwarz Waveform Relaxation                %
%                                                        %
%  C. Besse, IMT Toulouse, 2014                          %
%--------------------------------------------------------%

clear all; close all;

%% Donnees du probleme
a0=-5; b0=5;
T=1;
L=b0-a0;
N=1000;
dx=L/N;
Niter=100;
dt=T/Niter;

a=1;
nu=1;

x=(a0:dx:b0)';

overlap=50;

a1=a0;
b1=0+overlap*dx;
a2=0;
b2=b0;

x1=(a1:dx:b1)';
x2=(a2:dx:b2)';

%% Donnee initiale
phi=exp(-x.^2);

%% Terme source
f=0.5*(x>-1).*(x<1);

%% Restriction des donnees aux deux domaines
phi1=phi(1:501+overlap);
phi2=phi(501:N+1);
f1=f(1:501+overlap);
f2=f(501:N+1);


%% Valeur des noeuds d'echange
N1=length(phi1);
N2=length(phi2);
p1=N1-overlap;
p2=1+overlap;


%% Assemblage des matrices

e=ones(N1,1);
Mt1=(2/dt)*spdiags([e],0,N1,N1);
Ma1=(0.5/dx)*spdiags([-e,e],[-1,1],N1,N1);
Mlap1=(1/dx^2)*spdiags([e,-2*e,e],[-1:1],N1,N1);
e=ones(N2,1);
Mt2=(2/dt)*spdiags([e],0,N2,N2);
Ma2=(0.5/dx)*spdiags([-e,e],[-1,1],N2,N2);
Mlap2=(1/dx^2)*spdiags([e,-2*e,e],[-1:1],N2,N2);

M1=Mt1+a*Ma1-nu*Mlap1;
M2=Mt2+a*Ma2-nu*Mlap2;

cl1=zeros(N1,1);
cl2=zeros(N2,1);

% Conditions aux limites du probleme global
% Dirichlet a gauche
gl=0;
M1(1,1)=1;
M1(1,2)=0;
cl1(1)=gl;

% Neumann a droite
gr=0;
M2(N2,N2)=2/dt+2*nu/dx^2;
M2(N2,N2-1)=-2*nu/dx^2;
cl2(N2)=(2*nu/dx-a)*gr;

% Conditions d'interface
M1(N1,N1)=1;
M1(N1,N1-1)=0;

M2(1,1)=1;
M2(1,2)=0;

Transm_12km1=zeros(Niter,1);
Transm_21km1=zeros(Niter,1);
Transm_12k=zeros(Niter,1);
Transm_21k=zeros(Niter,1);

[LL1,UU1]=lu(M1);
[LL2,UU2]=lu(M2);

for k=1:15
  phi1=phi(1:501+overlap);
  phi2=phi(501:N+1);

  % Resolution sur le domaine 1
  for j=1:Niter-1
    rhs1=f1+cl1+(2/dt)*phi1; 
    rhs1(1)=cl1(1);
    rhs1(N1)=Transm_21km1(j);
    y=LL1\rhs1;
    v=UU1\y;
    phi1=2*v-phi1; 
    Transm_12k(j)=v(p1);
  end

  % Resolution sur le domaine 2
  for j=1:Niter-1
    rhs2=f2+cl2+(2/dt)*phi2; 
    rhs2(1)=Transm_12km1(j);
    y=LL2\rhs2;
    v=UU2\y;
    phi2=2*v-phi2; 
    Transm_21k(j)=v(p2);
  end
  Transm_12km1=Transm_12k;
  Transm_21km1=Transm_21k;
  plot(x1,phi1,x2,phi2);
  title(strcat('Iteration ',num2str(k)));
  drawnow;
  pause(0.5);
end

