import numpy as np
import scipy.sparse as sp
from terme_source import *

def assemblage_dom_phys(m,dom_phys):
    """
    --------------------------------------------------------
      Assemblage des matrices de masse et de rigidite       
      et du second membre pour methode d'elements finis P1  
      suivant le domaine physique considere                                                      
      C. Besse, IMT Toulouse, 2018                          
    --------------------------------------------------------
    """
    test_ibc=m["TRIANGLES"][:,3]==dom_phys
    (ind,)=test_ibc.nonzero()

    M=sp.lil_matrix((m["nbNod"],m["nbNod"]),dtype=np.float64)
    R=sp.lil_matrix((m["nbNod"],m["nbNod"]),dtype=np.float64)
    rhs=np.zeros(m["nbNod"],dtype=np.float64)
    lambdA=np.array(([-1,1,0],[-1,0,1]),dtype=np.float64)
    #matrice de masse locale de reference
    Mlocref=np.array(([2,1,1],[1,2,1],[1,1,2]),dtype=np.float64)

    K=np.zeros(m["nbNod"],dtype=np.int16)
    
    for ll in range(ind.size):
        l=ind[ll]
        #reperer les points du triangle
        k1=m["TRIANGLES"][l,0] #correspond au point A1
        k2=m["TRIANGLES"][l,1] #correspond au point A2
        k3=m["TRIANGLES"][l,2] #correspond au point A3

        K[k1]=1; K[k2]=1; K[k3]=1;
        
        #relever les coordonnees des points
        x1=m["POS"][k1,0]; y1=m["POS"][k1,1]; #coordonnees du point A1
        x2=m["POS"][k2,0]; y2=m["POS"][k2,1]; #coordonnees du point A2
        x3=m["POS"][k3,0]; y3=m["POS"][k3,1]; #coordonnees du point A3

        #calcul de l'aire du triangle
        aire=np.linalg.det(np.array(([x2-x1,x3-x1],[y2-y1,y3-y1]),dtype=np.float64))/2.

        #creation de la matrice de rigidite locale
        Kloc=np.zeros((3,3),dtype=np.float64)
        J=np.array(([y3-y1,y1-y2],[x1-x3,x2-x1]),dtype=np.float64)
        for j in range(3):
            Kloc[0,j]=np.dot(J.dot(lambdA[0:2,j]),J.dot(lambdA[0:2,0]))
            Kloc[1,j]=np.dot(J.dot(lambdA[0:2,j]),J.dot(lambdA[0:2,1]))
            Kloc[2,j]=np.dot(J.dot(lambdA[0:2,j]),J.dot(lambdA[0:2,2]))

        Kloc=1./(4.*aire)*Kloc

        #remplissage de la matrice de rigidite
        R[k1,k1]=R[k1,k1]+Kloc[0,0];    R[k1,k2]=R[k1,k2]+Kloc[0,1];    R[k1,k3]=R[k1,k3]+Kloc[0,2];
        R[k2,k1]=R[k2,k1]+Kloc[1,0];    R[k2,k2]=R[k2,k2]+Kloc[1,1];    R[k2,k3]=R[k2,k3]+Kloc[1,2];
        R[k3,k1]=R[k3,k1]+Kloc[2,0];    R[k3,k2]=R[k3,k2]+Kloc[2,1];    R[k3,k3]=R[k3,k3]+Kloc[2,2];

        #creation de la matrice de masse locale
        Mloc=(aire/12)*Mlocref
    
        #remplissage de la matrice de masse
        M[k1,k1]=M[k1,k1]+Mloc[0,0];    M[k1,k2]=M[k1,k2]+Mloc[0,1];    M[k1,k3]=M[k1,k3]+Mloc[0,2];
        M[k2,k1]=M[k2,k1]+Mloc[1,0];    M[k2,k2]=M[k2,k2]+Mloc[1,1];    M[k2,k3]=M[k2,k3]+Mloc[1,2];
        M[k3,k1]=M[k3,k1]+Mloc[2,0];    M[k3,k2]=M[k3,k2]+Mloc[2,1];    M[k3,k3]=M[k3,k3]+Mloc[2,2];
        
        #remplissage du second membre
        rhsloc=Mloc.dot(np.array(([fun_s(x1,y1)],[fun_s(x2,y2)],[fun_s(x3,y3)]),dtype=np.float64))
        rhs[[k1,k2,k3]]=rhs[[k1,k2,k3]]+rhsloc.transpose()

    for kk in range(m["nbNod"]):
        if not(K[kk]):
            R[kk,kk]=1
            M[kk,kk]=1
                
    return M,R,rhs

def cond_lim_dom_phys2(m,dom_phys,A,p):
    """
    --------------------------------------------------------
      Prise en compte des conditions aux limites liees      
      aux conditions de transmission                        
                                                            
      Methode de Schwarz Waveform Relaxation                
                                                            
      C. Besse, IMT Toulouse, 2018                          
    --------------------------------------------------------
    """
    test_ibc=m["LINES"][:,2]==dom_phys
    (ind_rep_bc_dom,)=test_ibc.nonzero()

    B=A.copy()
  
    for ll in range(ind_rep_bc_dom.size):
        l=ind_rep_bc_dom[ll]
        #reperer les points du segment du bord
        k1=m["LINES"][l,0]; #correspond au point A1
        k2=m["LINES"][l,1]; #correspond au point A2

        #relever les coordonnees des points
        x1=m["POS"][k1,0]; y1=m["POS"][k1,1]; #coordonnees du point A1
        x2=m["POS"][k2,0]; y2=m["POS"][k2,1]; #coordonnees du point A2

        #calcul de la longueur du segment
        long=np.sqrt((x2-x1)**2+(y2-y1)**2)

        # creation de la matrice de masse locale
        Mloc=(long/6)*np.array(([2,1],[1,2]),dtype=np.float64)
    
        #remplissage de la matrice de masse
        B[k1,k1]=B[k1,k1]+p*Mloc[0,0];    B[k1,k2]=B[k1,k2]+p*Mloc[0,1];    
        B[k2,k1]=B[k2,k1]+p*Mloc[1,0];    B[k2,k2]=B[k2,k2]+p*Mloc[1,1];    
        
    return B

def update_dnu2(m,dom_phys,p,u,v,dnv):
    """
    --------------------------------------------------------
      Mise a jour de la dreivee normale au bord du domaine  
      identifie par dom_phys                                
                                                            
      Methode de Schwarz Waveform Relaxation                
                                                            
      C. Besse, IMT Toulouse, 2018                          
    --------------------------------------------------------
    """
    ids=m["LINES"][:,2]==dom_phys
    (ind_rep_bc_dom,)=ids.nonzero()

    dnu=np.zeros(dnv.size,dtype=np.float64)
    mlocref=np.array(([2,1],[1,2]),dtype=np.float64)
  
    for ll in range(ind_rep_bc_dom.size):
        l=ind_rep_bc_dom[ll]
        #reperer les points du segment du bord
        k1=m["LINES"][l,0] #correspond au point A1
        k2=m["LINES"][l,1] #correspond au point A2
        
        #relever les coordonnees des points
        x1=m["POS"][k1,0]; y1=m["POS"][k1,1] #coordonnees du point A1
        x2=m["POS"][k2,0]; y2=m["POS"][k2,1] #coordonnees du point A2
        
        #calcul de la longueur du segment
        long=np.sqrt((x2-x1)**2+(y2-y1)**2)
        
        # creation de la matrice de masse locale
        Mloc=(long/6)*mlocref
        
        #remplissage du second membre
        dnu[[k1,k2]]=-p*Mloc.dot(np.array(([u[k1],u[k2]])))+dnv[[k1,k2]]+p*Mloc.dot(np.array(([v[k1],v[k2]])))
        
    return dnu
