import numpy as np
import scipy.sparse as sp
from terme_source import *

def cond_lim_dirichlet(m,A,rhs,CL,zones):
    """
    --------------------------------------------------------
      Prise en compte des conditions aux limites de         
      Dirichlet                                             
                                                            
      C. Besse, IMT Toulouse, 2018                        
    --------------------------------------------------------
     Determination des points du bords 
     Dans ls structure GMSH, les points du bord composent les lignes au bord
     mais ne sont pas forcement ranges dans un ordre pre-defini
    """
    test_ibc=m["LINES"][:,2]==zones[0]
    for k in range(1,len(zones)):
        test_ibc=np.logical_or(test_ibc , m["LINES"][:,2]==zones[k])

    (ibc,)=test_ibc.nonzero()
  
    noeud_bord=np.zeros((m["nbLines"],2),dtype=np.int64)
    vu=np.zeros(m["nbNod"],dtype=np.int32)
    k1=0
    for jj in range(len(ibc)):
        j=ibc[jj]
        p1=m["LINES"][j,0]
        p2=m["LINES"][j,1]
        mytype=m["LINES"][j,2]    
        if not(vu[p1]):
            noeud_bord[k1,0]=p1
            noeud_bord[k1,1]=mytype
            k1=k1+1
            vu[p1]=k1

        if not(vu[p2]):
            noeud_bord[k1,0]=p2
            noeud_bord[k1,1]=mytype
            k1=k1+1
            vu[p2]=k1

    vec_cl=np.zeros(m["nbNod"],dtype=np.float64)
    val_CL=CL;
    for j in range(k1):
        p=noeud_bord[j,0]
        mytype=noeud_bord[j,1]
        # if max(type==zones):
        #     # if (type==1001)or(type==1003)or(type==1004):
        #     # Test utile si on avait plusieurs zones differentes au bord
        #     val_CL=CL

        vec_cl[p]=val_CL
        # On multiplie la p-ieme colonne de la matrice par la valeur CL, et on
        # la soustrait au second membre. Le but est de laisser la matrice
        # symetrique
        rhs=rhs-A.dot(vec_cl)
        vec_cl[p]=0
        # La p-ieme ligne et la p-ieme colonne de la matrice sont remplacees
        # par une ligne  et une colonne de zeros.
        A[p,:]=0;A[:,p]=0;
        # Le terme App est remplacee par 1.
        A[p,p]=1;
        # La composante rhs(p) est remplacee par CL.
        rhs[p]=val_CL;

    return A,rhs

def cond_lim_dom_phys2(m,dom_phys,A,p):
    """
    --------------------------------------------------------
      Prise en compte des conditions aux limites liees      
      aux conditions de transmission                        
                                                            
      Methode de Schwarz Waveform Relaxation                
                                                            
      C. Besse, IMT Toulouse, 2018                          
    --------------------------------------------------------
    """
    test_ibc=m["LINES"][:,2]==dom_phys
    (ind_rep_bc_dom,)=test_ibc.nonzero()

    B=A.copy()
  
    for ll in range(ind_rep_bc_dom.size):
        l=ind_rep_bc_dom[ll]
        #reperer les points du segment du bord
        k1=m["LINES"][l,0]; #correspond au point A1
        k2=m["LINES"][l,1]; #correspond au point A2

        #relever les coordonnees des points
        x1=m["POS"][k1,0]; y1=m["POS"][k1,1]; #coordonnees du point A1
        x2=m["POS"][k2,0]; y2=m["POS"][k2,1]; #coordonnees du point A2

        #calcul de la longueur du segment
        long=np.sqrt((x2-x1)**2+(y2-y1)**2)

        # creation de la matrice de masse locale
        Mloc=(long/6)*np.array(([2,1],[1,2]),dtype=np.float64)
    
        #remplissage de la matrice de masse
        B[k1,k1]=B[k1,k1]+p*Mloc[0,0];    B[k1,k2]=B[k1,k2]+p*Mloc[0,1];    
        B[k2,k1]=B[k2,k1]+p*Mloc[1,0];    B[k2,k2]=B[k2,k2]+p*Mloc[1,1];    
        
    return B

def cond_lim_rhs(m,dom_phys,rhs,p,u,dnu):
    """
    --------------------------------------------------------
      Prise en compte des conditions aux limites liees      
      aux conditions de transmission pour le second         
      membre                                                
                                                            
      Methode de Schwarz Waveform Relaxation                
                                                            
      C. Besse, IMT Toulouse, 2018                          
    --------------------------------------------------------
    """
    ids=m["LINES"][:,2]==dom_phys
    (ind_rep_bc_dom,)=ids.nonzero()

    rhs_out=rhs.copy()
    mlocref=np.array(([2,1],[1,2]),dtype=np.float64)
  
    for ll in range(ind_rep_bc_dom.size):
        l=ind_rep_bc_dom[ll]
        #reperer les points du segment du bord
        k1=m["LINES"][l,0] #correspond au point A1
        k2=m["LINES"][l,1] #correspond au point A2
        
        #relever les coordonnees des points
        x1=m["POS"][k1,0]; y1=m["POS"][k1,1] #coordonnees du point A1
        x2=m["POS"][k2,0]; y2=m["POS"][k2,1] #coordonnees du point A2
        
        #calcul de la longueur du segment
        long=np.sqrt((x2-x1)**2+(y2-y1)**2)
        
        # creation de la matrice de masse locale
        Mloc=(long/6)*mlocref
        
        #remplissage du second membre
        rhs_out[[k1,k2]]=rhs_out[[k1,k2]]+p*Mloc.dot(np.array(([u[k1],u[k2]])))+dnu[[k1,k2]]

    return rhs_out
