import numpy as np
def load_gmsh(filename):
    # Contructing the Method structure
    msh={}
    # number of nodes in function of the element type
    msh["NODES_PER_TYPE_OF_ELEMENT"] = np.array([ 2,  3,  4,  4,  8,  6,  5,  3,  6,  9, 10, 27, 18, 14,  1,  8, 20, 15, 13 ],np.int32)
    # The format 2 don't sort the elements by reg phys but by
    # reg-elm. If this classification is important for your program,
    # use this (after calling this function):
    #
    # [OldRowNumber, NewRowNumber] = sort(OldMatrix(:,SortColumn)); 
    # NewMatrix = OldMatrix(NewRowNumber,:);
    #
    # Change the name of OldMatrix and NewMatrix with the name of yours
    # SortColumn by the number of the last column
    mesh = {}
    mesh["MIN"] = np.zeros((3, 1))
    mesh["MAX"] = np.zeros((3, 1))
    
    fid = open(filename, 'r')
    while 1:
        endoffile = 0
        while 1:
            tline = fid.readline()
            if (""==tline):
                endoffile=1
                break
            if (tline[0] == '$' ):
                if (tline[1] == 'N') and (tline[2] == 'O'):
                    fileformat = 1
                    break
                if (tline[1] == 'M') and (tline[2] == 'e'):
                    fileformat = 2
                    tline = fid.readline()
                    tline = fid.readline()
                    if (tline[0] == '$' and tline[1] == 'E' and tline[2] == 'n'):
                        tline = fid.readline()
                        break
                    else :
                        print (' This program can only read ASCII mesh file')
                        print (' of format 1 or 2 from GMSH, try again?')
                        endoffile=1
                        break
                if tline[1] == 'E' and (tline[2] == 'L' or tline[2] == 'l' ):
                    break
        if endoffile == 1 :
            break
        if tline[1] == 'N' and ( tline[2] == 'O' or tline[2] == 'o' ):
            #print('reading nodes')
            myline=fid.readline()
            mydata=myline.split()
            mesh["nbNod"] = int(mydata[0])
            mesh["POS"] = np.zeros((mesh["nbNod"], 3))
            IDS=np.zeros(mesh["nbNod"],np.int32)
            X=np.zeros(3)
            for i in range(mesh["nbNod"]):
                # iNod = fscanf(fid, '%d', 1);
                # X = fscanf(fid, '%g', 3);
                myline=fid.readline()
                mydata=myline.split()
                iNod = int(mydata[0])
                for j in range(3):
                    X[j]=float(mydata[1+j])
                IDS[iNod-1] = i
                if (i==0):
                    mesh["MIN"]=X.copy()
                    mesh["MAX"]=X.copy()
                else:
                    if(mesh["MAX"][0] < X[0]): mesh["MAX"][0] = X[0]
                    if(mesh["MAX"][1] < X[1]): mesh["MAX"][1] = X[1]
                    if(mesh["MAX"][2] < X[2]): mesh["MAX"][2] = X[2]
                    if(mesh["MIN"][0] > X[0]): mesh["MIN"][0] = X[0]
                    if(mesh["MIN"][1] > X[1]): mesh["MIN"][1] = X[1]
                    if(mesh["MIN"][2] > X[2]): mesh["MIN"][2] = X[2]
                mesh["POS"][i, 0] = X[0];
                mesh["POS"][i, 1] = X[1];
                mesh["POS"][i, 2] = X[2];
            tline=fid.readline()
            #print('nodes have been read')
        elif tline[1] == 'E' and ( tline[2] == 'L' or tline[2] == 'l') :
            #print('reading elements')
            # mesh.nbElm = fscanf(fid, '%d', 1);
            myline=fid.readline()
            mydata=myline.split()
            mesh["nbElm"]=int(mydata[0])
            if (fileformat == 1):
                nbinfo = 5
                tags = 3
            if (fileformat == 2):
                nbinfo = 4
                tags = 4
            mesh["ELE_INFOS"] = np.zeros((mesh["nbElm"], nbinfo),np.int32)
            mesh["nbPoints"] = 0
            mesh["nbLines"] = 0
            mesh["nbTriangles"] = 0
            mesh["nbQuads"] = 0
            mesh["nbTets"] = 0
            mesh["nbHexas"] = 0
            mesh["nbPrisms"] = 0
            mesh["nbPyramids"] = 0
            # comment next 8 lines to get "tight" arrays (will slow down reading)
            mesh["POINTS"] = np.zeros((mesh["nbElm"], 2),np.int32)
            mesh["LINES"] = np.zeros((mesh["nbElm"], 3),np.int32)
            mesh["TRIANGLES"] = np.zeros((mesh["nbElm"], 4),np.int32)
            mesh["QUADS"] = np.zeros((mesh["nbElm"], 5),np.int32)
            mesh["TETS"] = np.zeros((mesh["nbElm"], 5),np.int32)
            mesh["HEXAS"] = np.zeros((mesh["nbElm"], 9),np.int32)
            mesh["PRISMS"] = np.zeros((mesh["nbElm"], 7),np.int32)
            mesh["PYRAMIDS"] = np.zeros((mesh["nbElm"], 6),np.int32)
            for i in range(mesh["nbElm"]):
                # mesh.ELE_INFOS(I, :) = fscanf(fid, '%d', nbinfo);
                myline=fid.readline()
                mydata=myline.split()
                jref=0
                for j in range(nbinfo):
                    mesh["ELE_INFOS"][i,j]=int(mydata[j])
                jref=j+1
                if (fileformat == 1):
                    # take the mesh.ELE_INFOS(I, 5) nodes of the element
                    NODES_ELEM = np.zeros(mesh["ELE_INFOS"][i, 4],np.int32)
                    for j in range(mesh["ELE_INFOS"][i, 4]):
                        NODES_ELEM[j]=int(mydata[j+jref])
                if (fileformat == 2):
                    jref=jref+mesh["ELE_INFOS"][i,2]-1
                    mylabel = mesh["ELE_INFOS"][i,1]
                    NODES_ELEM = np.zeros(msh["NODES_PER_TYPE_OF_ELEMENT"][mylabel-1],np.int32)
                    for j in range(msh["NODES_PER_TYPE_OF_ELEMENT"][mylabel-1]):
                        NODES_ELEM[j] = int(mydata[j+jref])

                if (mesh["ELE_INFOS"][i, 1] == 15) : ## point
                    mesh["POINTS"][mesh["nbPoints"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["POINTS"][mesh["nbPoints"], 1] = mesh["ELE_INFOS"][i,tags-1]
                    mesh["nbPoints"] = mesh["nbPoints"] + 1

                if (mesh["ELE_INFOS"][i, 1] == 1) : ## line
                    mesh["LINES"][mesh["nbLines"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["LINES"][mesh["nbLines"], 1] = IDS[NODES_ELEM[1]-1]   
                    mesh["LINES"][mesh["nbLines"], 2] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbLines"] = mesh["nbLines"] + 1
      
                if (mesh["ELE_INFOS"][i, 1] == 2) : ## triangle
                    mesh["TRIANGLES"][mesh["nbTriangles"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["TRIANGLES"][mesh["nbTriangles"], 1] = IDS[NODES_ELEM[1]-1]
                    mesh["TRIANGLES"][mesh["nbTriangles"], 2] = IDS[NODES_ELEM[2]-1]
                    mesh["TRIANGLES"][mesh["nbTriangles"], 3] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbTriangles"] = mesh["nbTriangles"] + 1
      
                if (mesh["ELE_INFOS"][i, 1] == 3) : ## quadrangle
                    mesh["QUADS"][mesh["nbQuads"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["QUADS"][mesh["nbQuads"], 1] = IDS[NODES_ELEM[1]-1]
                    mesh["QUADS"][mesh["nbQuads"], 2] = IDS[NODES_ELEM[2]-1]
                    mesh["QUADS"][mesh["nbQuads"], 3] = IDS[NODES_ELEM[3]-1]
                    mesh["QUADS"][mesh["nbQuads"], 4] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbQuads"] = mesh["nbQuads"] + 1
      
                if (mesh["ELE_INFOS"][i, 1] == 4) : ## tetrahedron
                    mesh["TETS"][mesh["nbTets"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["TETS"][mesh["nbTets"], 1] = IDS[NODES_ELEM[1]-1]
                    mesh["TETS"][mesh["nbTets"], 2] = IDS[NODES_ELEM[2]-1]
                    mesh["TETS"][mesh["nbTets"], 3] = IDS[NODES_ELEM[3]-1]
                    mesh["TETS"][mesh["nbTets"], 4] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbTets"] = mesh["nbTets"] + 1
      
                if (mesh["ELE_INFOS"][i, 1] == 5) : ## hexahedron
                    mesh["HEXAS"][mesh["nbHexas"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 1] = IDS[NODES_ELEM[1]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 2] = IDS[NODES_ELEM[2]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 3] = IDS[NODES_ELEM[3]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 4] = IDS[NODES_ELEM[4]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 5] = IDS[NODES_ELEM[5]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 6] = IDS[NODES_ELEM[6]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 7] = IDS[NODES_ELEM[7]-1]
                    mesh["HEXAS"][mesh["nbHexas"], 8] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbHexas"] = mesh["nbHexas"] + 1
      
                if (mesh["ELE_INFOS"][i, 1] == 6) : ## prism
                    mesh["PRISMS"][mesh["nbPrisms"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["PRISMS"][mesh["nbPrisms"], 1] = IDS[NODES_ELEM[1]-1]
                    mesh["PRISMS"][mesh["nbPrisms"], 2] = IDS[NODES_ELEM[2]-1]
                    mesh["PRISMS"][mesh["nbPrisms"], 3] = IDS[NODES_ELEM[3]-1]
                    mesh["PRISMS"][mesh["nbPrisms"], 4] = IDS[NODES_ELEM[4]-1]
                    mesh["PRISMS"][mesh["nbPrisms"], 5] = IDS[NODES_ELEM[5]-1]
                    mesh["PRISMS"][mesh["nbPrisms"], 6] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbPrisms"] = mesh["nbPrisms"] + 1
      
                if (mesh["ELE_INFOS"][i, 1] == 7) : ## pyramid
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 0] = IDS[NODES_ELEM[0]-1]
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 1] = IDS[NODES_ELEM[1]-1]
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 2] = IDS[NODES_ELEM[2]-1]
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 3] = IDS[NODES_ELEM[3]-1]
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 4] = IDS[NODES_ELEM[4]-1]
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 5] = IDS[NODES_ELEM[5]-1]
                    mesh["PYRAMIDS"][mesh["nbPyramids"], 6] = mesh["ELE_INFOS"][i, tags-1]
                    mesh["nbPyramids"] = mesh["nbPyramids"] + 1
            tline = fid.readline()
            #print('elements have been read')
    fid.close()
    return mesh

                    




            

