#--------------------------------------------------------#
#  Resolution de l'equation de Poisson par methode       #
#                d'elements finis                        #
#                                                        #
#  Conditions aux limites de Dirichlet homogenes         #
#                                                        #
#  C. Besse, IMT Toulouse, 2018                          #
#--------------------------------------------------------#
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import scipy.sparse.linalg as slin
from load_gmsh import *
from assemblage import *
from cond_lim import *
plt.ion()

CL=0
# Identification des bords physiques ou imposer les conditions
# aux limites de Dirichlet
Bord1=1001
Bord2=1002
Bord3=1003
Bord4=1004

# Chargement du maillage
m=load_gmsh('Maillages/ef_glob.msh')
#m=load_gmsh('Maillages/cercle.msh')

# Assemblage des matrices de masse, de rigite
# et du second membre (depend de la fonction fun_s.m)
M,R,rhs=assemblage(m)

# Matrice du systeme lineaire.
A=R.copy()

# Prise en compte des conditions aux limites de Dirichlet sur les bords
# du domaine
# Pour le maillage "ef_glob.msh"
A,rhs=cond_lim_dirichlet(m,A,rhs,CL,np.array([Bord1,Bord2,Bord3,Bord4],dtype=np.int64))
# Pour le maillage "cercle.msh"
# A,rhs=cond_lim_dirichlet(m,A,rhs,CL,np.array([1001,2001],dtype=np.int64))

# Resolution du systeme lineaire
A=A.tocsc()
solve = slin.factorized(A)

sol=solve(rhs)

# figure(1)
xx=m["POS"][:,0]
yy=m["POS"][:,1]
triang=m["TRIANGLES"][0:m["nbTriangles"],0:3]
fig = plt.figure()
ax = fig.gca(projection='3d')
ax.plot_trisurf(xx,yy,sol,triangles=triang,cmap=plt.cm.viridis)
#ax.plot_trisurf(xx,yy,sol,triangles=triang,cmap=plt.cm.plasma)
