#--------------------------------------------------------#
#  Resolution de l'equation de Poisson par methode       #
#                d'elements finis                        #
#                                                        #
#  Conditions aux limites de Dirichlet homogenes         #
#                                                        #
#  Resolution par decomposition de domaines              #
#  Methode de Schwarz                                    #
#                                                        #
#  C. Besse, IMT Toulouse, 2018                          #
#--------------------------------------------------------#

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import scipy.sparse.linalg as slin
from load_gmsh import *
from trace import *
from assemblage import *
from assemblage_swr import *
from cond_lim import *

plt.ion()
plt.close('all')

m=load_gmsh('Maillages/ef2.msh') # Carre maillage grossier
#m=load_gmsh('Maillages/ef.msh') # Carre maillage fin
#m=load_gmsh('Maillages/cercle.msh')

param=10

## Representation des maillages pour les deux domaines
fig = plt.figure(1)
ax = fig.gca(projection='3d')
p1=trace_maillage_phys(m,1000,ax)
p2=trace_maillage_phys(m,2000,ax)
p1.set_edgecolors([0,0,1])
p2.set_edgecolors([1,0,0])
ax.view_init(elev=65,azim=15)

u1n=np.zeros(m["nbNod"])
u2n=u1n.copy()

# Zone d'echange frontiere :
# reperes 1002 (dom 1) et 2004 (dom 2)
ids=m["LINES"][:,2]==1002
(ind_rep_bc_dom1,)=ids.nonzero()
ids=m["LINES"][:,2]==2004
(ind_rep_bc_dom2,)=ids.nonzero()

# Preparation des premieres valeurs a la frontiere de transmission
# 0 ou aleatoire
for jj in range(ind_rep_bc_dom1.size):
  P1=m["LINES"][ind_rep_bc_dom1[jj],0]
  P2=m["LINES"][ind_rep_bc_dom1[jj],1]
  u1n[[P1,P2]]=0*np.random.rand(2,)

for jj in range(ind_rep_bc_dom2.size):
  P1=m["LINES"][ind_rep_bc_dom2[jj],0]
  P2=m["LINES"][ind_rep_bc_dom2[jj],1]
  u2n[[P1,P2]]=0*np.random.rand(2,)


## Trace des solutions sur les deux domaines
## En bleu, domaine 1, en rouge, domaine 2
fig2= plt.figure(2)
ax2 = fig2.gca(projection='3d')
p12=trace_sol_phys(m,u1n,1000,ax2)
p22=trace_sol_phys(m,u2n,2000,ax2)
p12.set_edgecolors([0,0,1])
p22.set_edgecolors([1,0,0])
p12.set_linewidth(0.5)
p22.set_linewidth(0.5)

## Preparation des tableaux pour le stockage des 
## derivees normales
dnu1n=np.zeros(m["nbNod"])
dnu2n=np.zeros(m["nbNod"])


# Assemblage des matrices de masse, de rigite
# et du second membre (depend de la fonction fun_s.m)
M1,R1,rhs1=assemblage_dom_phys(m,1000)
#A1=M1+R1
A1=R1.copy()
    
#import matplotlib
#matplotlib.pyplot.spy(A1,markersize=1)

# Assemblage des matrices de masse, de rigite
# et du second membre (depend de la fonction fun_s.m)
M2,R2,rhs2=assemblage_dom_phys(m,2000)
#A2=M2+R2
A2=R2.copy()

# Prise en compte des conditions aux limites de Dirichlet
A1,rhs1=cond_lim_dirichlet(m,A1,rhs1,0,[1001,1003,1004])
A2,rhs2=cond_lim_dirichlet(m,A2,rhs2,0,[2001,2002,2003])

# Prise en compte des conditions de transmission aux interfaces
A1=cond_lim_dom_phys2(m,1002,A1,param)
A2=cond_lim_dom_phys2(m,2004,A2,param)

A1=A1.tocsc(); solve1 = slin.factorized(A1)
A2=A2.tocsc(); solve2 = slin.factorized(A2)


for jjj in range(20):
    dnu1=dnu1n.copy()
    dnu2=dnu2n.copy()
    u1=u1n.copy()
    u2=u2n.copy()
    rhs1b=cond_lim_rhs(m,1002,rhs1,param,u2, dnu2)
    rhs2b=cond_lim_rhs(m,2004,rhs2,param,u1,-dnu1)

    u1n=solve1(rhs1b)
    u2n=solve2(rhs2b)

    dnu1n=update_dnu2(m,1002, param,u1n,u2,dnu2)
    dnu2n=update_dnu2(m,2004,-param,u2n,u1,dnu1)

    ax2.remove()
    ax2 = fig2.gca(projection='3d')
    p12=trace_sol_phys(m,u1n,1000,ax2)
    p22=trace_sol_phys(m,u2n,2000,ax2)
    p12.set_edgecolors([0,0,1])
    p22.set_edgecolors([1,0,0])
    p12.set_linewidth(0.5)
    p22.set_linewidth(0.5)

    print('Iteration ',jjj+1)
    plt.pause(0.5)

