#--------------------------------------------------------#
#  Resolution de l'equation d'advection diffusion        #
#                                                        #
#  \partial_t u + a\partial_x u -\nu \partial_x^2 u =f   #
#  Conditions aux limites de Dirichlet a gauche,         #
#  Neumann a droite                                      #
#                                                        #
#  C. Besse, IMT Toulouse, 2018                          #
#--------------------------------------------------------#

import numpy as np
import matplotlib.pylab as plt
import scipy.sparse as sp
import scipy.sparse.linalg as slin
plt.ion()


## Donnees du probleme
a0=-5; b0=5
T=1
L=b0-a0
N=1000
dx=L/N
x=np.arange(a0,b0+dx,dx)
Niter=100
dt=T/Niter

a=1
nu=1


### Numbers of plots during the simulation
Sav=20
Sa=max(np.floor(Niter/Sav),1)

## Assemblage de la matrice
vec   = np.ones((N+1));
data  = (2/dt)*np.array([vec]); diags = np.array([0])
Mt    = sp.spdiags(data,diags,N+1,N+1, format='lil')
data  = (0.5/dx)*np.array([-vec,vec]); diags = np.array([-1,1])
Ma    = sp.spdiags(data,diags,N+1,N+1, format='lil')
data  = (1/dx**2)*np.array([vec,-2*vec,vec]); diags = np.array([-1,0,1])
Mlap  = sp.spdiags(data,diags,N+1,N+1, format='lil')

M=Mt+a*Ma-nu*Mlap

## Donnee initiale
phi=np.exp(-x**2)

## Terme source
f=0.5*(x>-1)*(x<1)

# Conditions aux limites
cl=np.zeros(N+1)

# Dirichlet a gauche
gl=0
M[0,0]=1
M[0,1]=0
cl[0]=gl

# Neumann a droite
gr=0
M[N,N]=2/dt+2*nu/dx**2
M[N,N-1]=-2*nu/dx**2
cl[N]=(2*nu/dx-a)*gr

M=M.tocsc()
solve = slin.factorized(M)

plt.close('all')
fig = plt.figure(1)
line1 = plt.Line2D([], [], color='blue', linewidth=1)
ax=plt.subplot(1,1,1)
ax.add_line(line1)
ax.set_xlim([a0, b0])
ax.set_ylim([0, 1.2])
#ax.set_aspect('equal', 'datalim')
ax.set_xlabel('x') # Setting x-axis label
ax.set_ylabel('u') # Setting y-axis label
line1.set_data(x[:], phi[:])
plt.draw()
plt.show()


# Resolution du probleme temporel
for j in range(Niter):
  rhs=f+cl+(2/dt)*phi; rhs[0]=cl[0]
  v=solve(rhs)
  phi=2*v-phi; phi[0]=gl

  if (j%Sa==0):
      line1.set_data(x[:], phi[:])
      plt.draw()
      plt.show()
      plt.pause(0.005);
